      subroutine accumfunc(task,iobs,id,scaleweight,obsweight,nfull,
     *   nfree,plusmin,fulltype,iffull,fullbeta,dfullbeta,dfreebeta,
     *   lnlall,lnlobs,fp,fpt,sp,spt,nchol,chol,ifdrv,ipt,jpt,warn)
 
c     Subroutine to accumulate likelihood and derivatives for each
c     observation.  Second derivatives are approximated as minus the
c     outerproduct of first derivatives.  Note that only the lower
c     triangle of the second derivatives matrix is computed; it needs
c     to be expanded after all observations have been handled.
c
c     fp = first derivatives wrt true parameters
c     fpt  = first derivatives wrt transformed parameters (e.g.,
c            a standard deviation must be positive, so we search on
c            log(sigma), a transformed parameter.)
c     sp = second derivatives wrt true parameters
c     spt  = second derivatives wrt transformed parameters
c     chol = one or more "compressed" matrices with Cholesky
c            decompositions of covariance matrices that are searched
c            on using a Cholesky decomposition.  If nchol=0,
c            there are no such matrices.
c
c     There may be episodes that enter the denominator, i.e.,
c     "observations" that are subtracted from the likelihood.
c     If plusmin=0, this observation stands alone, i.e., can be
c           processed as if denominator observations don't exist;
c        plusmin=1, this observation is followed by a negative
c           observation;
c        plusmin=-1, this observation is subtracted.
c        plusmin=-2, there was nothing in the numerator.
c     See subroutine figure_model.
c
c     Constantijn Panis,  29 August 2002
c     Last change: iffull integer
 
      implicit none
      integer task,iobs,id,nfull,nfree,plusmin,fulltype(nfull),
     *   iffull(nfull),nchol,ipt,jpt,warn,i,j,numunder,pos,dim,
     *   cmpdim,intcontig
      double precision fullbeta(nfull),dfullbeta(nfull),
     *   dfreebeta(nfree),lnlall,lnlobs,fp(nfree),fpt(nfree),
     *   sp(nfree,nfree),spt(nfree,nfree),chol(nchol),
     *   weight,dpnan,minexp
      real scaleweight,obsweight
      logical ifdrv,isnan
      save numunder
 
c     If the function is NaN, derivatives must also be.
      if(isnan(lnlobs))then
         lnlall=dpnan()
         if(mod(task/10,10).eq.1.or.mod(task/10,10).eq.2)
     *      call put(dpnan(),fp,nfree)
         if(mod(task/10,10).eq.1)call put(dpnan(),fpt,nfree)
         if(mod(task,10).eq.1.or.mod(task,10).eq.2)
     *      call put(dpnan(),sp,nfree*nfree)
         if(mod(task,10).eq.1)call put(dpnan(),spt,nfree*nfree)
         goto 999
      endif
 
c     Did this observation underflow?
c     NOTE: to carefully address this issue, we may want to store all
c     lnL for all observations, so that underflowing observations may
c     be set to the lnL of a previous iteration.  Also, we need to
c     account for "observations" that are in the denominator of the
c     likelihood.
      if(iobs.eq.1)numunder=0
      if(warn.ne.0.and.lnlobs.eq.minexp())then
         numunder=numunder+1
         if(ipt.ge.3)then
            if(numunder.eq.1)write(8,'(1x)')
            if(numunder.le.20)call writeaia(8,72,1,
     *         '*** WARNING: likelihood underflows to zero for ID = ',
     *         id,' ***')
            if(numunder.eq.21)write(8,102)
         endif
         if(jpt.ge.3)then
            if(numunder.eq.1)write(6,'(1x)')
            if(numunder.le.20)call writeaia(6,72,1,
     *         '*** WARNING: likelihood underflows to zero for ID = ',
     *         id,' ***')
            if(numunder.eq.21)write(6,102)
         endif
  102    format('*** WARNING: no more warnings for likelihood ',
     *      'underflows will be printed ***')
      endif
 
      weight = dble(scaleweight*obsweight)
      if(plusmin.eq.0)then
c        Standard case: numerator only.
         if(ifdrv)call dpfull2free(dfullbeta,dfreebeta,iffull,nfull)
         if(task/100.eq.1)lnlall = lnlall + weight*lnlobs
      elseif(plusmin.eq.1)then
c        Denominator observation to follow; cut short.
         if(ifdrv)call dpfull2free(dfullbeta,dfreebeta,iffull,nfull)
         if(task/100.eq.1)lnlall = lnlall + weight*lnlobs
         goto 999
      elseif(plusmin.le.-1)then
         if(ifdrv)then
            if(plusmin.eq.-2)call zerodp(nfree,dfreebeta)
            j=0
            do 100 i=1,nfull
               if(iffull(i).eq.0)goto 100
               j=j+1
               dfreebeta(j)=dfreebeta(j)-dfullbeta(i)
  100       continue
         endif
         if(task/100.eq.1)lnlall = lnlall - weight*lnlobs
      endif
 
      if(.not.ifdrv)goto 999
 
c     Accumulate derivatives wrt untransformed parameters:
      call accumfpsp(nfree,dfreebeta,fp,sp,weight,
     *   (mod(task/10,10).eq.1.or.mod(task/10,10).eq.2),
     *   (mod(task,10).eq.1.or.mod(task,10).eq.2))
 
c     If derivatives wrt transformed parameters are not desired, exit.
      if(mod(task/10,10).eq.2.or.mod(task,10).eq.2)goto 999
 
c     Compute first derivatives wrt transformed parameters:
      if(nchol.gt.0)then
c        There is a search on Cholesky decomposed parameters.
c        Convert relevant part of dfreebeta into derivatives wrt
c        Cholesky decomposition.
         pos=1
         j=0
         i=0
  200    i=i+1
         if(i.gt.nfull)goto 300
         if(iffull(i).eq.0)goto 200
         j=j+1
         if(fulltype(i).ne.5)goto 200
         dim=intcontig(5,fulltype(i),nfull+1-i)
         cmpdim=(dim*(dim+1))/2
c        use dfullbeta as scratch array:
         call copy8(dfreebeta(j),dfullbeta,cmpdim)
c        Convert derivatives wrt standard deviations and correlations
c        into wrt the Cholesky decomposition:
         call chol2corr(2,dim,chol(pos),fullbeta(i),dfullbeta,
     *      dfreebeta(j))
         pos=pos+cmpdim
         i=i+cmpdim-1
         j=j+cmpdim-1
         goto 200
      endif
c     Convert (rest of) derivatives wrt untransformed parameters into
c     wrt transformed parameters
  300 call fp2fpt(nfull,nfree,dfreebeta,dfreebeta,fullbeta,fulltype,
     *   iffull)
c     Accumulate derivatives wrt transformed parameters:
      call accumfpsp(nfree,dfreebeta,fpt,spt,weight,
     *   (mod(task/10,10).eq.1),mod(task,10).eq.1)
 
  999 return
      end
