      double precision function digama(x)
 
C Inputs:
C       X, double precision
C Outputs:
C       function returns double precision
C Function:
C        Computes derivative of the log(gamma(x)) function.
C Notes:
C        Accurate to 20 decimals.
C        By William Cody, from netlib.specfun.
C       Optimized by Constantijn Panis, 3 February 1992.
C       Also note dlgama, which computes log(gamma).
C*
C----------------------------------------------------------------------
C
C This function program evaluates the logarithmic derivative of the
C   gamma function,
C
C      digama(x) = d/dx (gamma(x)) / gamma(x) = d/dx (ln gamma(x))
C
C   for real x, where either
C
C          -xmax1 < x < -xmin (x not a negative integer), or
C            xmin < x.
C
C   The calling sequence for this function is
C
C                  Y = digama(X)
C
C   The main computation uses rational Chebyshev approximations
C   published in Math. Comp. 27, 123-127 (1973) by Cody, Strecok and
C   Thacher.  This transportable program is patterned after the
C   machine-dependent FUNPACK program digama(X), but cannot match that
C   version for efficiency or accuracy.  This version uses rational
C   approximations that are theoretically accurate to 20 significant
C   decimal digits.  The accuracy achieved depends on the arithmetic
C   system, the compiler, the intrinsic functions, and proper selection
C   of the machine-dependent constants.
C
C*******************************************************************
C*******************************************************************
C
C Explanation of machine-dependent constants
C
C   XINF   = largest positive machine number
C   XMAX1  = beta ** (p-1), where beta is the radix for the
C            floating-point system, and p is the number of base-beta
C            digits in the floating-point significand.  This is an
C            upper bound on non-integral floating-point numbers, and
C            the negative of the lower bound on acceptable negative
C            arguments for digama.  If rounding is necessary, round this
C            value down.
C   XMIN1  = the smallest in magnitude acceptable argument.  We
C            recommend XMIN1 = MAX(1/XINF,xmin) rounded up, where
C            xmin is the smallest positive floating-point number.
C   XSMALL = absolute argument below which  PI*COTAN(PI*X)  may be
C            represented by 1/X.  We recommend XSMALL < sqrt(3 eps)/pi,
C            where eps is the smallest positive number such that
C            1+eps > 1.
C   XLARGE = argument beyond which digama(X) may be represented by
C            LOG(X).  The solution to the equation
C               x*ln(x) = beta ** p
C            is a safe value.
C
C     Approximate values for some important machines are
C
C                        beta  p     eps     xmin       XINF
C
C  CDC 7600      (S.P.)    2  48  7.11E-15  3.13E-294  1.26E+322
C  CRAY-1        (S.P.)    2  48  7.11E-15  4.58E-2467 5.45E+2465
C  IEEE (IBM/XT,
C    SUN, etc.)  (S.P.)    2  24  1.19E-07  1.18E-38   3.40E+38
C  IEEE (IBM/XT,
C    SUN, etc.)  (D.P.)    2  53  1.11D-16  2.23E-308  1.79D+308
C  IBM 3033      (D.P.)   16  14  1.11D-16  5.40D-79   7.23D+75
C  SUN 3/160     (D.P.)    2  53  1.11D-16  2.23D-308  1.79D+308
C  VAX 11/780    (S.P.)    2  24  5.96E-08  2.94E-39   1.70E+38
C                (D.P.)    2  56  1.39D-17  2.94D-39   1.70D+38
C   (G Format)   (D.P.)    2  53  1.11D-16  5.57D-309  8.98D+307
C
C                         XMIN1      XMAX1     XSMALL    XLARGE
C
C  CDC 7600      (S.P.)  3.13E-294  1.40E+14  4.64E-08  9.42E+12
C  CRAY-1        (S.P.)  1.84E-2466 1.40E+14  4.64E-08  9.42E+12
C  IEEE (IBM/XT,
C    SUN, etc.)  (S.P.)  1.18E-38   8.38E+06  1.90E-04  1.20E+06
C  IEEE (IBM/XT,
C    SUN, etc.)  (D.P.)  2.23D-308  4.50D+15  5.80D-09  2.71D+14
C  IBM 3033      (D.P.)  1.39D-76   4.50D+15  5.80D-09  2.05D+15
C  SUN 3/160     (D.P.)  2.23D-308  4.50D+15  5.80D-09  2.71D+14
C  VAX 11/780    (S.P.)  5.89E-39   8.38E+06  1.35E-04  1.20E+06
C                (D.P.)  5.89D-39   3.60D+16  2.05D-09  2.05D+15
C   (G Format)   (D.P.)  1.12D-308  4.50D+15  5.80D-09  2.71D+14
C
C*******************************************************************
C*******************************************************************
C
C Error Returns
C
C  The program returns XINF for  X < -XMAX1, for X zero or a negative
C    integer, or when X lies in (-XMIN1, 0), and returns -XINF
C    when X lies in (0, XMIN1).
C
C Intrinsic functions required are:
C
C     ABS, AINT, DBLE, INT, LOG, REAL, TAN
C
C
C  Author: W. J. Cody
C          Mathematics and Computer Science Division
C          Argonne National Laboratory
C          Argonne, IL 60439
C
C  Latest modification: June 8, 1988
C
C----------------------------------------------------------------------
 
      implicit none
      integer nq
      double precision aug,den,piov4,
     *   sgn,xlarge,upper,w,x,xinf,xmax1,xmin1,xsmall,x01,
     *   x01d,x02,y,z
      double precision p11,p12,p13,p14,p15,p16,p17,p18,p19,
     *   p21,p22,p23,p24,p25,p26,p27,
     *   q11,q12,q13,q14,q15,q16,q17,q18,
     *   q21,q22,q23,q24,q25,q26
c----------------------------------------------------------------------
c     mathematical constants.  piov4 = pi / 4
c----------------------------------------------------------------------
      data piov4/7.8539816339744830962d-01/
c----------------------------------------------------------------------
c     machine-dependent constants
c----------------------------------------------------------------------
      data xinf/1.79d+308/,
     *    xmin1/2.23d-308/,
     *    xmax1/4.50d+15/,
     *   xsmall/5.80d-09/,
     *   xlarge/2.71d+14/
c----------------------------------------------------------------------
c     zero of digama(x)
c----------------------------------------------------------------------
      data x01/187.0d0/,x01d/128.0d0/,x02/6.9464496836234126266d-04/
c----------------------------------------------------------------------
c     coefficients for approximation to  digama(x)/(x-x0)  over [0.5, 3.0]
c----------------------------------------------------------------------
      data p11,p12,p13,p14,p15,p16,p17,p18,p19/
     *   4.5104681245762934160d-03,5.4932855833000385356d+00,
     *   3.7646693175929276856d+02,7.9525490849151998065d+03,
     *   7.1451595818951933210d+04,3.0655976301987365674d+05,
     *   6.3606997788964458797d+05,5.8041312783537569993d+05,
     *   1.6585695029761022321d+05/
      data q11,q12,q13,q14,q15,q16,q17,q18/
     *   9.6141654774222358525d+01,2.6287715790581193330d+03,
     *   2.9862497022250277920d+04,1.6206566091533671639d+05,
     *   4.3487880712768329037d+05,5.4256384537269993733d+05,
     *   2.4242185002017985252d+05,6.4155223783576225996d-08/
c----------------------------------------------------------------------
c     coefficients for approximation to  digama(x) - ln(x) + 1/(2x)
c     for  x > 3.0
c----------------------------------------------------------------------
      data p21,p22,p23,p24,p25,p26,p27/
     *   -2.7103228277757834192d+00,-1.5166271776896121383d+01,
     *   -1.9784554148719218667d+01,-8.8100958828312219821d+00,
     *   -1.4479614616899842986d+00,-7.3689600332394549911d-02,
     *   -6.5135387732718171306d-21/
      data q21,q22,q23,q24,q25,q26/
     *   4.4992760373789365846d+01,2.0240955312679931159d+02,
     *   2.4736979003315290057d+02,1.0742543875702278326d+02,
     *   1.7463965060678569906d+01,8.8427520398873480342d-01/
 
      y = x
      w = abs(y)
      aug = 0.d0
c----------------------------------------------------------------------
c     check for valid arguments, then branch to appropriate algorithm
c----------------------------------------------------------------------
      if((-y.ge.xmax1).or.(w.lt.xmin1))then
         goto 410
      elseif(y.ge..5d0)then
         goto 200
c----------------------------------------------------------------------
c     y < 0.5, use reflection formula:
c           digama(1-y) = digama(y) + pi * cot(pi*y)
c     use 1/y for pi*cotan(pi*y)  when  xmin1 < |y| <= xsmall.
c----------------------------------------------------------------------
      elseif(w.le.xsmall)then
         aug = -1.d0 / y
         goto 150
      endif
c----------------------------------------------------------------------
c     argument reduction for cot
c----------------------------------------------------------------------
  100 if(y.lt.0.d0)then
         sgn = piov4
      else
         sgn = -piov4
      endif
      w = w - aint(w)
      nq = int(w * 4.d0)
      w = 4.d0 * (w - dble(nq) * .25d0)
c     The following statement will never execute but is necessary because
c     Digital Fortran's optimizer screws up without it.
      if(w.eq.1.d6)write(*,*)'This should never happen'
c----------------------------------------------------------------------
c     w is now related to the fractional part of  4.0 * y.
c     adjust argument to correspond to values in the first
c     quadrant and determine the sign.
c----------------------------------------------------------------------
      if(mod(nq,2).ne.0)w = 1.d0 - w
      z = piov4 * w
      if(mod(nq/2,2).ne.0)sgn = - sgn
c----------------------------------------------------------------------
c     determine the final value for  -pi * cotan(pi*x)
c----------------------------------------------------------------------
      if(mod((nq+1)/2,2).eq.0)then
c----------------------------------------------------------------------
c     check for singularity
c----------------------------------------------------------------------
         if(z.eq.0.d0)goto 410
         aug = sgn * (4.d0 / tan(z))
      else
         aug = sgn * (4.d0 * tan(z))
      endif
  150 y = 1.d0 - y
  200 if(y.gt.3.d0)goto 300
c----------------------------------------------------------------------
c     0.5 <= y <= 3.0
c----------------------------------------------------------------------
      den = (((((((y+q11)*y+q12)*y+q13)*y+q14)*y+q15)*y+q16)*y+q17)*y
      upper = (((((((p11*y+p12)*y+p13)*y+p14)*y+p15)*y+p16)*y+p17)*y+
     *   p18)*y
      den = (upper + p19) / (den + q18)
      y = (y-x01/x01d) - x02
      digama = den * y + aug
      goto 999
c----------------------------------------------------------------------
c     3.0 < y
c----------------------------------------------------------------------
  300 if(y.lt.xlarge)then
         w = 1.d0 / (y * y)
         den = (((((w+q21)*w+q22)*w+q23)*w+q24)*w+q25)*w
         upper = (((((p21*w+p22)*w+p23)*w+p24)*w+p25)*w+p26)*w
         aug = (upper + p27) / (den + q26) - .5d0 / y + aug
      endif
      digama = aug + log(y)
      goto 999
c----------------------------------------------------------------------
c  error return
c----------------------------------------------------------------------
  410 digama = xinf
      if(y.gt.0.d0)digama = -xinf
 
  999 return
      end
 
 
