      double precision function dlgama(x)
 
C Inputs:
C       X, double precision
C Outputs:
C       function returns double precision.
C Function:
C        Computes log(gamma(x)).
C Notes:
C        Accurate to 18 decimals.
C        By William Cody, from netlib.specfun.
C       Optimized by Constantijn Panis, 3 February 1992
C       Also note digama, which computes derivatives of dlgama.
C*
C----------------------------------------------------------------------
C
C This routine calculates the LOG(GAMMA) function for a positive real
C   argument X.  Computation is based on an algorithm outlined in
C   references 1 and 2.  The program uses rational functions that
C   theoretically approximate LOG(GAMMA) to at least 18 significant
C   decimal digits.  The approximation for X > 12 is from reference
C   3, while approximations for X < 12.0 are similar to those in
C   reference 1, but are unpublished.  The accuracy achieved depends
C   on the arithmetic system, the compiler, the intrinsic functions,
C   and proper selection of the machine-dependent constants.
C
C
C**************************************************************
C
C Error returns
C
C  The program returns the value XINF for X .LE. 0.0 or when
C     overflow would occur.  The computation is believed to
C     be free of underflow and overflow.
C
C
C Intrinsic functions required are:
C
C      LOG
C
C  Latest modification: June 16, 1988
C
C----------------------------------------------------------------------
      implicit none
      double precision corr,d1,d2,d4,eps,frtbig,four,half,one,pnt68,
     *   res,sqrtpi,thrhal,twelve,two,x,xbig,xden,xinf,
     *   xm1,xm2,xm4,xnum,y,ysq,zero
      double precision c1,c2,c3,c4,c5,c6,c7,
     *   p11,p12,p13,p14,p15,p16,p17,p18,
     *   p21,p22,p23,p24,p25,p26,p27,p28,
     *   p41,p42,p43,p44,p45,p46,p47,p48,
     *   q11,q12,q13,q14,q15,q16,q17,q18,
     *   q21,q22,q23,q24,q25,q26,q27,q28,
     *   q41,q42,q43,q44,q45,q46,q47,q48
c----------------------------------------------------------------------
c  mathematical constants
c----------------------------------------------------------------------
      data one,half,twelve,zero/1.0d0,0.5d0,12.0d0,0.0d0/,
     *   four,thrhal,two,pnt68/4.0d0,1.5d0,2.0d0,0.6796875d0/,
     *   sqrtpi/0.9189385332046727417803297d0/
c----------------------------------------------------------------------
c  machine dependent parameters
c----------------------------------------------------------------------
      data xbig,xinf,eps,frtbig/2.55d305,1.79d308,2.22d-16,2.25d76/
c----------------------------------------------------------------------
c  numerator and denominator coefficients for rational minimax
c     approximation over (0.5,1.5).
c----------------------------------------------------------------------
      data d1/-5.772156649015328605195174d-1/
      data p11,p12,p13,p14,p15,p16,p17,p18/
     *   4.945235359296727046734888d0,2.018112620856775083915565d2,
     *   2.290838373831346393026739d3,1.131967205903380828685045d4,
     *   2.855724635671635335736389d4,3.848496228443793359990269d4,
     *   2.637748787624195437963534d4,7.225813979700288197698961d3/
      data q11,q12,q13,q14,q15,q16,q17,q18/
     *   6.748212550303777196073036d1,1.113332393857199323513008d3,
     *   7.738757056935398733233834d3,2.763987074403340708898585d4,
     *   5.499310206226157329794414d4,6.161122180066002127833352d4,
     *   3.635127591501940507276287d4,8.785536302431013170870835d3/
c----------------------------------------------------------------------
c  numerator and denominator coefficients for rational minimax
c     approximation over (1.5,4.0).
c----------------------------------------------------------------------
      data d2/4.227843350984671393993777d-1/
      data p21,p22,p23,p24,p25,p26,p27,p28/
     *   4.974607845568932035012064d0,5.424138599891070494101986d2,
     *   1.550693864978364947665077d4,1.847932904445632425417223d5,
     *   1.088204769468828767498470d6,3.338152967987029735917223d6,
     *   5.106661678927352456275255d6,3.074109054850539556250927d6/
      data q21,q22,q23,q24,q25,q26,q27,q28/
     *   1.830328399370592604055942d2,7.765049321445005871323047d3,
     *   1.331903827966074194402448d5,1.136705821321969608938755d6,
     *   5.267964117437946917577538d6,1.346701454311101692290052d7,
     *   1.782736530353274213975932d7,9.533095591844353613395747d6/
c----------------------------------------------------------------------
c  numerator and denominator coefficients for rational minimax
c     approximation over (4.0,12.0).
c----------------------------------------------------------------------
      data d4/1.791759469228055000094023d0/
      data p41,p42,p43,p44,p45,p46,p47,p48/
     *   1.474502166059939948905062d4,2.426813369486704502836312d6,
     *   1.214755574045093227939592d8,2.663432449630976949898078d9,
     *   2.940378956634553899906876d10,1.702665737765398868392998d11,
     *   4.926125793377430887588120d11,5.606251856223951465078242d11/
      data q41,q42,q43,q44,q45,q46,q47,q48/
     *   2.690530175870899333379843d3,6.393885654300092398984238d5,
     *   4.135599930241388052042842d7,1.120872109616147941376570d9,
     *   1.488613728678813811542398d10,1.016803586272438228077304d11,
     *   3.417476345507377132798597d11,4.463158187419713286462081d11/
c----------------------------------------------------------------------
c  coefficients for minimax approximation over (12, inf).
c----------------------------------------------------------------------
      data c1,c2,c3,c4,c5,c6,c7/
     *   -1.910444077728d-03,8.4171387781295d-04,
     *   -5.952379913043012d-04,7.93650793500350248d-04,
     *   -2.777777777777681622553d-03,8.333333333333333331554247d-02,
     *   5.7083835261d-03/
 
      y = x
      if((y.gt.zero).and.(y.le.xbig))then
         if(y.le.eps)then
            res = -log(y)
         elseif(y.le.thrhal)then
c----------------------------------------------------------------------
c     eps < x <= 1.5
c----------------------------------------------------------------------
            if(y.lt.pnt68)then
               corr = -log(y)
               xm1 = y
            else
               corr = zero
               xm1 = y-one
            endif
            if((y.le.half).or.(y.ge.pnt68))then
               xnum = ((((((p11*xm1+p12)*xm1+p13)*xm1 +
     *            p14)*xm1+p15)*xm1+p16)*xm1+p17)*xm1+p18
               xden = (((((((xm1+q11)*xm1+q12)*xm1+q13)*xm1 +
     *            q14)*xm1+q15)*xm1+q16)*xm1+q17)*xm1+q18
               res = corr + (xm1 * (d1 + xm1*(xnum/xden)))
            else
               xm2 = y-one
               xnum = ((((((p21*xm2+p22)*xm2+p23)*xm2 +
     *            p24)*xm2+p25)*xm2+p26)*xm2+p27)*xm2+p28
               xden = (((((((xm2+q21)*xm2+q22)*xm2+q23)*xm2 +
     *            q24)*xm2+q25)*xm2+q26)*xm2+q27)*xm2+q28
               res = corr + xm2 * (d2 + xm2*(xnum/xden))
            endif
         elseif(y.le.four)then
c----------------------------------------------------------------------
c     1.5 < x <= 4
c----------------------------------------------------------------------
            xm2 = y - two
            xnum = ((((((p21*xm2+p22)*xm2+p23)*xm2 +
     *         p24)*xm2+p25)*xm2+p26)*xm2+p27)*xm2+p28
            xden = (((((((xm2+q21)*xm2+q22)*xm2+q23)*xm2 +
     *         q24)*xm2+q25)*xm2+q26)*xm2+q27)*xm2+q28
            res = xm2 * (d2 + xm2*(xnum/xden))
         elseif(y.le.twelve)then
c----------------------------------------------------------------------
c     4 < x <= 12
c----------------------------------------------------------------------
            xm4 = y - four
            xnum = ((((((p41*xm4+p42)*xm4+p43)*xm4 +
     *         p44)*xm4+p45)*xm4+p46)*xm4+p47)*xm4+p48
            xden = (((((((-xm4+q41)*xm4+q42)*xm4+q43)*xm4 +
     *         q44)*xm4+q45)*xm4+q46)*xm4+q47)*xm4+q48
            res = d4 + xm4*(xnum/xden)
         else
c----------------------------------------------------------------------
c     x >= 12
c----------------------------------------------------------------------
            res = zero
            if(y.le.frtbig)then
               ysq = y * y
               res = (((((c7/ysq+c1)/ysq+c2)/ysq+c3)/ysq +
     *            c4)/ysq+c5)/ysq+c6
            endif
            res = res/y
            corr = log(y)
            res = res + sqrtpi - half*corr
            res = res + y*(corr-one)
         endif
      else
c----------------------------------------------------------------------
c     return for bad arguments
c----------------------------------------------------------------------
         res = xinf
      endif
c----------------------------------------------------------------------
c     final adjustments and return
c----------------------------------------------------------------------
      dlgama = res
 
      return
      end
