      subroutine dnormk4(lnprob,k,x,sigma,mu,ifdrv,ifsigma,dsigma,
     *   ifmu,dmu,xminmu,xmmsi,status)
 
c     Subroutine to compute a (log of a) k-dimensional normal density.
c     Arguments:
c     lnprob   [out]  logarithm of normal density
c     k        [in]   dimension of distribution
c     x        [in]   points in k-dimensional space
c     sigma    [in]   covariance matrix of stochast x.
c                     NOTE: will be destroyed upon return!!
c                     As a potentially useful service, sigma will
c                     contain its own inverse upon return.
c     mu       [in]   mean of stochast x
c     ifdrv    [in]   want derivatives?
c                     NOTE: the computation of the derivatives is very
c                     straightforward.  Checking ifmu and ifsigma would
c                     be relatively costly; skipping it may on average
c                     be more efficient and has the advantage that ifmu
c                     and ifsigma need not be computed from their
c                     original components.
c     dsigma   [scratch & out]  derivatives of lnprob wrt sigma
c     dmu      [out]  derivatives of lnprob wrt mu
c     xminmu   [scratch] will contain x-mu upon return
c     xmmsi    [scratch] will contain (x-mu)'*sigmainv upon return
c     status   [out]  0 if OK; 1 if sigma is singular
c
c     This routine differs from dnormk in that the outcome x is in
c     single precision.
c
c     Constantijn Panis, 3 December 2002
c     Last change: log-determinant
 
      implicit none
      integer status,k,i,j
      double precision twopi
      parameter(twopi=6.283185307179586476925287d0)
      double precision lnprob,sigma(k,k),mu(k),dsigma(k,k),
     *   dmu(k),sign,lndet,xminmu(k),xmmsi(k),xmmsixmm
      real x(k)
      logical ifdrv,ifsigma,ifmu
 
      status=0
c     compute inverse and determinant of sigma:
c     Note that subroutine nrminv will destroy sigma.
c     Note that dsigma will be used as scratch array for sigmainv!!
      call nrminv(2,sigma,dsigma,k,sign,lndet,status)
      if(status.gt.0.or.sign.le.0.d0)goto 999
      call copy8(dsigma,sigma,k*k)
 
c     compute x-mu:
      do 100 i=1,k
         xminmu(i) = dble(x(i))-mu(i)
  100 continue
 
c     compute (x-mu)'*sigmainv:
      call m2mult(100,1,k,k,xminmu,sigma,xmmsi,0)
 
c     compute the quadratic form (x-mu)'*sigmainv*(x-mu)
      call m2mult(000,1,k,1,xmmsi,xminmu,xmmsixmm,0)
 
c     compute the log-likelihood:
c        constant - .5*ln(determinant) -.5(x-mu)'*sigmainv*(x-mu),
c     where constant = -.5*k*ln(2*pi)
      lnprob = -.5d0*(k*dlog(twopi) + lndet + xmmsixmm)
 
c     compute derivatives (if desired):
      if(.not.ifdrv)goto 999
 
c     derivatives wrt mu happen to be xmmsi:
      if(ifmu)call copy8(xmmsi,dmu,k)
 
c     derivatives wrt sigma:
c     derivative of ln(det) wrt a parameter is
c     trace(sigmainv*derivative of sigma wrt that parameter);
c     if the parameter is element (i,j) of sigma, this collapses to
c     element (j,i) of sigmainv = element (i,j) of sigmainv.
c     derivative of xmmsixmm wrt a parameter is
c     -xminmu'*sigmainv*(deriv of sigma wrt parameter)*sigmainv*xminmu;
c     if the parameter is element (i,j) of sigma, this collapses to
c     -xmmsi(i)*xmmsi(j)
      if(.not.ifsigma)goto 999
      do 300 i=1,k
         do 310 j=1,i
            dsigma(i,j) = .5d0*(-sigma(i,j) + xmmsi(i)*xmmsi(j))
            dsigma(j,i) = dsigma(i,j)
  310    continue
  300 continue
 
  999 return
      end
