      subroutine eigen(task,a,r,n)
 
c     Subroutine to compute eigenvalues and eigenvectors of a real
c     symmetric matrix.
c
c     Arguments:
c     task [in]  0   compute eigenvalues and eigenvectors.
c                1   compute eigenvalues only
c     a    [in/out] double precision symmetric input matrix of dimension
c                   (n,n).  Note: matrix will be destroyed; the resultant
c                   eigenvalues are copied into the first n elements of a.
c                   The eigenvalues are stored in descending order.
c     r    [out] matrix of eigenvectors; stored columnwise, in same
c                sequence as eigenvectors.
c     n    [in]  dimension of a and r.
c
c     Method: diagonalization method originated by Jacobi and adapted
c     by Von Neumann for large computers as found in 'Mathematical
c     methods for digital computers', edited by A. Ralston and
c     H.S. Wilf, John Wiley and Sons, New York, 1962, chapter 7.
c
c     Note: this routine is derived from subroutine eigen as found in
c     the Maxlik package.  That subroutine required a special storage
c     mode for the input matrix and returned the eigenvalues in a
c     similar mode.  I adapted that -- the input matrix is a normal
c     (n,n) symmetric matrix, of which only the lower triangle is
c     used.  There is no check that the matrix is indeed symmetric.
c
c     Constantijn Panis,  7 November 1991
 
      implicit none
      integer task,n,i,j,iq,ij,ia,ind,l,m,lq,mq,lm,ll,mm,ilq,imq,
     *   ilr,imr,jq,il,im,k
      double precision a(n*n),r(n*n),range,anorm,anormx,thr,x,y,
     *   sinx,sinx2,cosx,cosx2,sincos
 
c     Change the storage mode (lower triangle is copied in first
c     n*(n+1)/2 elements).
      call my_mstr(a,a,n)
 
c     Generate identity matrix
    5 range=1.d-12
      if(task.eq.0)then
         iq=-n
         do 20 j=1,n
            iq=iq+n
            do 21 i=1,n
               ij=iq+i
               r(ij)=0.d0
               if(i.eq.j)r(ij)=1.d0
   21       continue
   20    continue
      endif
 
c     Compute initial and final norms (anorm and anormx)
      anorm=0.d0
      do 35 i=1,n
         do 36 j=i+1,n
            ia=i+(j*j-j)/2
            anorm=anorm+a(ia)*a(ia)
   36    continue
   35 continue
      if(anorm.le.0.d0)goto 165
      anorm=1.414d0*dsqrt(anorm)
      anormx=anorm*range/dble(n)
 
c     Initialize indicators and compute threshold, thr
      ind=0
      thr=anorm
   45 thr=thr/dble(n)
   50 l=1
   55 m=l+1
 
c     Compute sin and cos
   60 mq=(m*m-m)/2
      lq=(l*l-l)/2
      lm=l+mq
      if(dabs(a(lm)).lt.thr)goto 130
      ind=1
      ll=l+lq
      mm=m+mq
      x=0.5d0*(a(ll)-a(mm))
      y=-a(lm)/dsqrt(a(lm)*a(lm)+x*x)
      if(x.lt.0.d0)y=-y
      sinx=y/dsqrt(2.d0*(1.d0+(dsqrt(1.d0-y*y))))
      sinx2=sinx*sinx
      cosx=dsqrt(1.d0-sinx2)
      cosx2=cosx*cosx
      sincos=sinx*cosx
 
c     Rotate l and m columns
      ilq=n*(l-1)
      imq=n*(m-1)
      do 125 i=1,n
         iq=(i*i-i)/2
         if(i.eq.l)goto 115
         if(i.lt.m)then
            im=i+mq
         elseif(i.eq.m)then
            goto 115
         elseif(i.gt.m)then
            im=m+iq
         endif
         if(i.lt.l)then
            il=i+lq
         else
            il=l+iq
         endif
         x=a(il)*cosx-a(im)*sinx
         a(im)=a(il)*sinx+a(im)*cosx
         a(il)=x
  115    if(task.eq.1)goto 125
         ilr=ilq+i
         imr=imq+i
         x=r(ilr)*cosx-r(imr)*sinx
         r(imr)=r(ilr)*sinx+r(imr)*cosx
         r(ilr)=x
  125 continue
      x=2.d0*a(lm)*sincos
      y=a(ll)*cosx2+a(mm)*sinx2-x
      x=a(ll)*sinx2+a(mm)*cosx2+x
      a(lm)=(a(ll)-a(mm))*sincos+a(lm)*(cosx2-sinx2)
      a(ll)=y
      a(mm)=x
 
c     Tests for completion
c     Test for m = last column
  130 if(m.eq.n)goto 140
      m=m+1
      goto 60
 
c     Test for l = second from last column
  140 if(l.eq.n-1)goto 150
      l=l+1
      goto 55
  150 if(ind.eq.1)then
         ind=0
         goto 50
      endif
 
c     Compare threshold with final norm
      if(thr.gt.anormx)goto 45
 
c     Sort eigenvalues and eigenvectors
  165 iq=-n
      do 185 i=1,n
         iq=iq+n
         ll=i+(i*i-i)/2
         jq=n*(i-2)
         do 190 j=i,n
            jq=jq+n
            mm=j+(j*j-j)/2
            if(a(ll).ge.a(mm))goto 190
            x=a(ll)
            a(ll)=a(mm)
            a(mm)=x
            if(task.eq.1)goto 190
            do 180 k=1,n
               ilr=iq+k
               imr=jq+k
               x=r(ilr)
               r(ilr)=r(imr)
               r(imr)=x
  180       continue
  190    continue
  185 continue
 
c     Copy eigenvalues into first n elements of a:
      do 210 i=2,n
         a(i) = a(i*(i+1)/2)
  210 continue
 
      return
      end
c=======================================================================
      subroutine my_mstr(a,b,n)
 
c     Subroutine to copy the lower triangle of a matrix into the
c     first n*(n+1)/2 elements.
 
      implicit none
      integer n,i,j,k
      double precision a(n,n),b(n*(n+1)/2)
 
      k=0
      do 100 j=1,n
         do 200 i=1,j
            k=k+1
            b(k) = a(i,j)
  200    continue
  100 continue
 
      return
      end
 
 
