      subroutine ginverse(a,n1,n2,ainv,work,status)
 
c     Subroutine to compute a generalized inverse using a singular
c     value decomposition.  If A = U*W*V', then Ainv = V*Winv*U'.
c     (U is column orthogonal; W is diagonal with zero or positive
c     elements; V is orthogonal -- also see subroutine svd.)
c     Since W is diagonal, its inverse is simple.  There is one
c     problem: if A is of rank less than n2, some elements of W
c     are zero.  They need to be replace by 0, rather than 1/0 to
c     obtain Winv.  If diagonal elements of W are very close to zero,
c     the decision whether to replace them by 0 or 1/W(i,i) is
c     somewhat arbitrary.  Parameter eps determines the cutoff value:
c     if W(i,i)<eps then Winv(i,i)=0, else Winv(i,i)=1/W(i,i)
c
c     Cutoff value 1.d-6 was found to strike a good balance in terms
c     of the error of the 4 properties of a generalized inverse:
c     (1) A*Ainv is symmetric;
c     (2) Ainv*A is symmetric;
c     (3) A*Ainv*A = A;
c     (4) Ainv*A*Ainv = Ainv.
c
c     Arguments:
c     a      [in] arbitrary matrix of dimension (n1,n2), n1>=n2
c     ainv  [out] generalized inverse of a, dimension (n2,n1)
c     work  [scr] scratch space of dimension (n2,n2); upon output
c                 work contains V*Winv.
c     status [out] 0 if OK; 1 otherwise
c
c     Reference: "Numerical Recipes" by Press, Flannery, Teukolsky
c     and Vetterling (Cambridge University Press 1986), section 2.9,
c     pp 52-63.
c
c     Constantijn Panis,  23 March 2001
c     Last change: status instead of alternate return
 
      implicit none
      integer n1,n2,status,i2
      double precision a(n1,n2),ainv(n2,n1),work(n2,n2),eps
      parameter(eps=1.d-6)
 
      status=0
      if(n1.lt.n2)goto 800
 
      if(n1.eq.1)then
         if(a(1,1).eq.0.d0)then
            ainv(1,1) = 0.d0
         else
            ainv(1,1) = 1.d0/a(1,1)
         endif
         goto 999
      endif
 
c     NOTE: Ainv will be placed in matrix ainv, but it is only computed
c           as a last step and ainv can thus be used as scratch space.
c           It is used for both W and for RV1 in svd.  (Note that at
c           this stage, ainv is at least of dimension (1,2))
c           Scratch array work will be used for V, and later for V*W.
 
      call svd(a,n1,n2,ainv,work,ainv(1,2),status)
      if(status.gt.0)goto 800
c     A now really contains U.
 
c     Compute V*Winv and store in work
      do 100 i2=1,n2
         if(ainv(i2,1).le.eps)then
            call zerodp(n2,work(1,i2))
         else
            call mult(work(1,i2),1.d0/ainv(i2,1),work(1,i2),n2)
         endif
  100 continue
 
c     Compute Ainv = V*Winv*U'
      call m2mult(010,n2,n2,n1,work,a,ainv,0)
      goto 999
 
  800 status=1
  999 return
      end
