      subroutine hermit(task,dim,z,wt)
 
c     Subroutine to calculate the zeros z(i) of the dim-th order Hermite
c     polynomial.  The largest zero will be stored in z(1).  Also
c     calculates the corresponding coefficients wt(i) of the dim-th order
c     Gauss-Hermite quadrature formula of degree 2*dim-1.
c
c     Arguments:
c     task [in]  1=compute full set;   2=compute half only (rest symmetric)
c     dim  [in]  order of Hermite polynomial
c     z   [out]  Hermite zero solutions (points), in increasing order
c                (and all negative if task=2)
c     wt  [out]  corresponding coefficients (weights)
c
c     Constantijn Panis, 24 November 1995
 
      implicit none
      integer task,dim,i
c     If task=2, only z((dim+1)/2) and wt((dim+1)/2) are accessed:
      double precision z(dim),wt(dim),fn,cc,s,zt,dpn,pn1,dlgama
 
      fn=dble(dim)
      cc=1.7724538509d0*dexp(dlgama(fn))/(2.d0**(dim-1))
      s=(2.d0*fn+1.d0)**.16667d0
 
      do 100 i=1,(dim+1)/2
         if(i.eq.1)then
            zt=s**3-1.85575d0/s
         elseif(i.eq.2)then
            zt=zt-1.14d0*fn**.426d0/zt
         elseif(i.eq.3)then
            zt=1.86d0*zt+.86d0*z(1)
         elseif(i.eq.4)then
            zt=1.91d0*zt+.91d0*z(2)
         else
            zt=2.d0*zt+z(i-2)
         endif
 
         call hroot(zt,dim,dpn,pn1,0.d0)
         z(i)=-zt
         wt(i)=cc/dpn/pn1
c        Zeroes and corresponding coefficients are symmetric:
         if(task.ne.1)goto 100
         z(dim-i+1)=-zt
         wt(dim-i+1)=wt(i)
  100 continue
 
      return
      end
c=================================================================
      subroutine hroot(z,dim,dpn,pn1,eps)
c
c         improves the approximate root z
c         in addition we also obtain
c
c             dpn = derivative of h(n) at z
c             pn1 = value of h(n-1) at z
c
      implicit none
      integer dim,iter
      double precision z,dpn,pn1,eps,p,dp,d
 
      iter=0
  100 iter=iter+1
      call hrecur(p,dp,pn1,z,dim)
      d=p/dp
      z=z-d
      if(abs(d).gt.eps.and.iter.lt.10)goto 100
      dpn=dp
 
      return
      end
c=================================================================
      subroutine hrecur(pn,dpn,pn1,z,dim)
      implicit none
      integer dim,j
      double precision pn,dpn,pn1,z,p1,fj,fj2,p,dp1,dp,q,dq
 
      p1=1.d0
      p=z
      dp1=0.d0
      dp=1.d0
      do 100 j=2,dim
        fj=dble(j)
        fj2=.5d0*(fj-1.d0)
        q=z*p-fj2*p1
        dq=z*dp+p-fj2*dp1
        p1=p
        p=q
        dp1=dp
        dp=dq
  100 continue
 
      pn=p
      dpn=dp
      pn1=p1
 
      return
      end
