      subroutine multiprobit(ne4,cmu4,eps,tauminax,sigma44,prob,ifdrv,
     *   dtau,dax,dsigma44,ipt,jpt,status)
 
c     Subroutine to compute the bi- or trivariate cumulative normal
c     distribution.  Both open and closed interval probabilities are
c     supported.  Arguments:
c     cmu4     [in] conditional mean of eps (conditional on realizations
c                   in continuous models)
c     eps      [in] heterogeneity draws
c     tauminax [in] probit thresholds.  Minus infinity ("less than")
c                   is denoted by -dpinf, plus infinity by dpinf.
c                   Two thresholds are not supported.
c     sigma44  [in] Correlation matrix of residual structure with
c                   standard errors on diagonal and correlations off-
c                   diagonally.
c                                                  tau - AX - CMu - eps
c     To be computed is Phi(A1,A2,rho), where A1 = --------------------,
c                                                       sigma
c     et cetera.   Derivatives are assigned, not accummulated.
c
c     Upon return, status is
c        0 if normal execution
c       -1 if the likelihood underflows to zero
c        1 if a standard deviation was zero (message is written)
c        4 if correlation matrix was not positive definite (message written)
c
c     Constantijn Panis, 22 February 2002
c     Last change: catch prob<0 due to rounding errors.  (See Eiji Mangyo's
c     floating point exception, reported on 2/11/2002.)
 
      implicit none
      integer ne4,ipt,jpt,status,i,j,interval
      double precision cmu4(ne4),eps(ne4),tauminax(2,ne4),prob,
     *   sigma44(ne4,ne4),a(2,3),dtau(2,ne4),dax(ne4),dsigma44(ne4,ne4),
     *   da(2,3),temp,temp1,temp2,dpinf
      logical ifdrv,isinf
 
      status=0
c     Convert probability into all "less than" statements and compute
c     normalized thresholds.  The lower triangle of sigma44 may be
c     reversed in sign; will be restored from upper triangle, below.
      interval=0
      do 100 i=1,ne4
         if(sigma44(i,i).eq.0.d0)goto 810
         if(isinf(tauminax(2,i)))then
            if(isinf(-tauminax(1,i)))call bug(ipt,jpt)
c           Convert "greater than" into "less than" by flipping
c           sign of threshold and signs of all covariances on
c           row i and column i (lower triangle only)
            a(1,i)=-dpinf()
            a(2,i)=-(tauminax(1,i)-cmu4(i)-eps(i))/sigma44(i,i)
            do 110 j=1,ne4
               if(j.lt.i)then
                  sigma44(i,j)=-sigma44(i,j)
               elseif(j.gt.i)then
                  sigma44(j,i)=-sigma44(j,i)
               endif
  110       continue
         else
            if(isinf(-tauminax(1,i)))then
               a(1,i)=-dpinf()
            else
               a(1,i)=(tauminax(1,i)-cmu4(i)-eps(i))/sigma44(i,i)
               interval=1
            endif
            a(2,i)=(tauminax(2,i)-cmu4(i)-eps(i))/sigma44(i,i)
         endif
  100 continue
 
c     Initialize probability and derivatives:
      prob=0.d0
      if(ifdrv)then
         call zerodp(2*ne4,da)
         call zerodp(ne4*ne4,dsigma44)
      endif
 
      if(ne4.eq.2)then
         call area2(2,2,a,sigma44,prob,ifdrv,da,dsigma44,1)
         if(interval.eq.0)goto 120
         call area2(1,2,a,sigma44,prob,ifdrv,da,dsigma44,-1)
         call area2(2,1,a,sigma44,prob,ifdrv,da,dsigma44,-1)
         call area2(1,1,a,sigma44,prob,ifdrv,da,dsigma44,1)
      elseif(ne4.eq.3)then
         status=0
         call volume3(2,2,2,a,sigma44,prob,ifdrv,da,dsigma44,1,status)
c        Beware of trivariate covariance matrix that is not positive definite
         if(status.ne.0)goto 820
         if(interval.eq.0)goto 120
         call volume3(1,2,2,a,sigma44,prob,ifdrv,da,dsigma44,-1,status)
         call volume3(2,1,2,a,sigma44,prob,ifdrv,da,dsigma44,-1,status)
         call volume3(2,2,1,a,sigma44,prob,ifdrv,da,dsigma44,-1,status)
         call volume3(1,1,2,a,sigma44,prob,ifdrv,da,dsigma44,1,status)
         call volume3(1,2,1,a,sigma44,prob,ifdrv,da,dsigma44,1,status)
         call volume3(2,1,1,a,sigma44,prob,ifdrv,da,dsigma44,1,status)
         call volume3(1,1,1,a,sigma44,prob,ifdrv,da,dsigma44,-1,status)
      else
         goto 800
      endif
 
c     Beware of underflows:
  120 if(prob.le.0.d0)then
         status=-1
         goto 999
      endif
 
      if(.not.ifdrv)then
c        Restore lower triangle of sigma44:
         do 200 i=1,ne4
            do 210 j=1,i-1
               sigma44(i,j)=sigma44(j,i)
  210       continue
  200    continue
         goto 999
      endif
 
c     Unravel wrt a into wrt tau, ax, and standard deviations.  Correct
c     signs of derivatives wrt correlations.  Also restore sigma44 to its
c     original form.  Note that derivatives wrt cmu4 and eps are same as
c     wrt ax, so they are not computed and stored separately.
      do 300 i=1,ne4
         temp2=da(2,i)/sigma44(i,i)
         dsigma44(i,i)=-a(2,i)*temp2
         if(isinf(tauminax(2,i)))then
            dtau(1,i)=-temp2
            dax(i)=dax(i)+temp2
            do 310 j=1,ne4
               if(j.lt.i)then
                  dsigma44(i,j)=-dsigma44(i,j)
               elseif(j.gt.i)then
                  dsigma44(j,i)=-dsigma44(j,i)
               endif
  310       continue
         else
            if(.not.isinf(-tauminax(1,i)))then
c              Bounded interval
               temp1=da(1,i)/sigma44(i,i)
               dtau(1,i)=temp1
               dax(i)=dax(i)-temp1
               dsigma44(i,i)=dsigma44(i,i)-a(1,i)*temp1
            endif
            dtau(2,i)=temp2
            dax(i)=dax(i)-temp2
         endif
  300 continue
 
c     Restore lower triangle of sigma44 and spread out derivatives
c     over both sides of the diagonal
      do 400 i=1,ne4
         do 410 j=1,i-1
            sigma44(i,j)=sigma44(j,i)
            temp=.5d0*dsigma44(i,j)
            dsigma44(i,j)=temp
            dsigma44(j,i)=temp
  410    continue
  400 continue
 
      goto 999
 
c     Only bi- and trivariate probits are supported:
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'ERROR:  only bi- and trivariate probit functions are ',
     *   'currently supported.',/,'Higher order probits must be ',
     *   'independent.  Your application requires a',/,
     *   'non-independent probit of order greater than three.')
      stop
  810 status=1
      if(ipt.gt.0)write(8,811)
      if(jpt.gt.0)write(6,811)
  811 format(/,'The function evaluation failed because the standard ',
     *   'deviation of a probit',/,'residual is zero.')
      goto 999
c     Trivariate correlation matrix not positive definite
  820 status=4
      if(ipt.gt.0)write(8,821)
      if(jpt.gt.0)write(6,821)
  821 format(/,'The function evaluation failed because the ',
     *   'correlation matrix of a',/,'trivariate probit is not ',
     *   'positive definite.')
      goto 999
  999 return
      end
c====================================================================
      subroutine area2(i,j,a,sigma,prob,ifdrv,da,dsigma,plus)
 
c     Subroutine to add or subtract a pnorm2 to/from prob, and
c     accumulate derivatives wrt thresholds and the correlation.
c
c     Constantijn Panis, 27 February 2002
c     Last change: analytical derivatives except wrt rho
 
      implicit none
      integer i,j,plus
      double precision a(2,2),sigma(2,2),prob,da(2,2),dsigma(2,2),
     *   x,y,rho,dx,dy,drho,det,delta,temp,
     *   pnorm2,pnorm1,dnorm1,jacobeps
      logical ifdrv,isinf
 
      x=a(i,1)
      y=a(j,2)
      if(isinf(-x).or.isinf(-y))goto 999
      rho=sigma(2,1)
      temp = pnorm2(x,y,rho)
      if(plus.eq.1)then
         prob = prob + temp
      else
         prob = prob - temp
      endif
      if(ifdrv)then
         det=1.d0-rho*rho
         dx = pnorm1((y-rho*x)/dsqrt(det))*dnorm1(x)
         dy = pnorm1((x-rho*y)/dsqrt(det))*dnorm1(y)
         delta = jacobeps(rho)
         if(rho+delta.ge.1.d0)delta=-delta
         drho = (pnorm2(x,y,rho+delta)-temp)/delta
         if(plus.eq.1)then
            da(i,1)=da(i,1)+dx
            da(j,2)=da(j,2)+dy
            dsigma(2,1)=dsigma(2,1)+drho
         else
            da(i,1)=da(i,1)-dx
            da(j,2)=da(j,2)-dy
            dsigma(2,1)=dsigma(2,1)-drho
         endif
      endif
 
  999 return
      end
c====================================================================
      subroutine volume3(i,j,k,a,sigma,prob,ifdrv,da,dsigma,plus,status)
 
c     Subroutine to add or subtract a pnorm3 to/from prob, and
c     accumulate derivatives wrt thresholds and correlations.
c
c     Constantijn Panis, 27 February 2002
c     Last change: analytical derivatives except wrt correlations.
 
      implicit none
      integer status,i,j,k,plus
      double precision a(2,3),sigma(3,3),prob,da(2,3),dsigma(3,3),
     *   x,y,z,rhoxy,rhoxz,rhoyz,sigmax,sigmay,sigmaz,rho,dx,dy,dz,
     *   drhoxy,drhoxz,drhoyz,delta,temp,pnorm3,pnorm2,dnorm1,jacobeps
      logical ifdrv,isinf
 
      x=a(i,1)
      y=a(j,2)
      z=a(k,3)
      if(isinf(-x).or.isinf(-y).or.isinf(-z))goto 999
      rhoxy=sigma(2,1)
      rhoxz=sigma(3,1)
      rhoyz=sigma(3,2)
      temp = pnorm3(x,y,z,rhoxy,rhoxz,rhoyz)
      if(temp.eq.-1.d0)then
c        Correlation matrix not positive definite
         status=3
         goto 999
      endif
      if(plus.eq.1)then
         prob = prob + temp
      else
         prob = prob - temp
      endif
      if(ifdrv)then
         sigmay=dsqrt(1.d0-rhoxy**2)
         sigmaz=dsqrt(1.d0-rhoxz**2)
         rho=(rhoyz-rhoxy*rhoxz)/(sigmay*sigmaz)
         dx = pnorm2((y-rhoxy*x)/sigmay,(z-rhoxz*x)/sigmaz,rho)*
     *      dnorm1(x)
         sigmax=dsqrt(1.d0-rhoxy**2)
         sigmaz=dsqrt(1.d0-rhoyz**2)
         rho=(rhoxz-rhoxy*rhoyz)/(sigmax*sigmaz)
         dy = pnorm2((x-rhoxy*y)/sigmax,(z-rhoyz*y)/sigmaz,rho)*
     *      dnorm1(y)
         sigmax=dsqrt(1.d0-rhoxz**2)
         sigmay=dsqrt(1.d0-rhoyz**2)
         rho=(rhoxy-rhoxz*rhoyz)/(sigmax*sigmay)
         dz = pnorm2((x-rhoxz*z)/sigmax,(y-rhoyz*z)/sigmay,rho)*
     *      dnorm1(z)
         delta = jacobeps(rhoxy)
         drhoxy = (pnorm3(x,y,z,rhoxy+delta,rhoxz,rhoyz)-temp)/delta
         delta = jacobeps(rhoxz)
         drhoxz = (pnorm3(x,y,z,rhoxy,rhoxz+delta,rhoyz)-temp)/delta
         delta = jacobeps(rhoyz)
         drhoyz = (pnorm3(x,y,z,rhoxy,rhoxz,rhoyz+delta)-temp)/delta
         if(plus.eq.1)then
            da(i,1)=da(i,1)+dx
            da(j,2)=da(j,2)+dy
            da(k,3)=da(k,3)+dz
            dsigma(2,1)=dsigma(2,1)+drhoxy
            dsigma(3,1)=dsigma(3,1)+drhoxz
            dsigma(3,2)=dsigma(3,2)+drhoyz
         else
            da(i,1)=da(i,1)-dx
            da(j,2)=da(j,2)-dy
            da(k,3)=da(k,3)-dz
            dsigma(2,1)=dsigma(2,1)-drhoxy
            dsigma(3,1)=dsigma(3,1)-drhoxz
            dsigma(3,2)=dsigma(3,2)-drhoyz
         endif
      endif
 
  999 return
      end
 
