      subroutine nrminv(task,x,xinv,n,sign,lndet,status)
 
c     General purpose subroutine to invert a matrix.  The algorithm follows
c     Numerical Recipes, page 38, 39, hence the name Numerical Recipes
c     Matrix Inversion.
c
c     Arguments:
c        task   [in] 1 general purpose matrix inversion only
c                    2 both invert matrix and return log-determinant
c                    3 compute log-determinant only
c        x(n,n) [in]  matrix to be inverted; is destroyed upon return!
c        xinv   [out] inverse of x
c        sign   [out] sign of determinant
c        lndet  [out] log-absolute-determinant of x
c                     If sign=-1, det=-exp(lndet)
c                              0, det=0
c                              1, det=exp(lndet)
c        status [out] status: 0 if all OK
c                             1 if error: matrix is singular
c
c     For documentation see Numerical Recipes, page 38, 39
c     Can't deal with more than (maxdim,maxdim);  maxdim=4000.
c     If maxdim is exceeded, a program bug report is generated.
c
c     Note the existence of subroutine definv, which inverts
c     positive and negative definite matrices considerably more
c     efficiently than this general purpose routine.
c
c     Constantijn Panis, 3 December 2002
c     Last change: log-determinant
 
      implicit none
      integer task,n,maxdim
      parameter(maxdim=4000)
      double precision x(n,n),xinv(n,n),vv(maxdim),sign,ss,lndet,dpmin
      integer status,i,indx(maxdim)
 
      status=0
      if(n.gt.maxdim)call bug(1,1)
 
      call ludcmp(x,n,indx,ss,vv,status)
      if(status.gt.0)then
c        LU decomposition was impossible, matrix is singular:
         if(task.le.2)goto 999
         goto 900
      endif
 
c     Compute inverse matrix:
      if(task.le.2)then
         call identity(n,xinv)
         do 100 i=1,n
            call lubksb(x,n,indx,xinv(1,i))
  100    continue
      endif
 
c     Compute determinant:
      if(task.ge.2)then
         sign=ss
         lndet=0.d0
         do 200 i=1,n
            if(x(i,i).eq.0.d0)goto 900
            if(x(i,i).lt.0.d0)sign=-sign
            lndet=lndet+dlog(dabs(x(i,i)))
  200    continue
c        The following statement neutralizes a bug in Digital Fortran's optimize
c        Without this statement, Shailender's Bangladeshi health model leads to
c        a supposed singular covariance matrix with 828 continuous outcomes.
c        (The bug existed when computing det; don't know about lndet.)
c        if(lndet.eq.-lndet+1.d0)write(6,*)lndet
      endif
      goto 999
 
  900 sign=0.d0
      lndet=-dpmin()
  999 return
      end
c=====================================================================
      subroutine ludcmp(a,n,indx,sign,vv,status)
C Inputs:
C        A(n,n) matrix
C Outputs:
C        A is replaced by its LU decomposition
C        Indx is an output vector which records the row permutation
C        effected by the partial pivoting.
C        D is output +1 or -1 if #interchanges was even or odd, resp.
C Function:
C        Compute LU decomposition of a rowwise permutation of a matrix.
C        Used to compute the inverse of a matrix (lubksb also needed)
C        or to compute its determinant.
C Notes:
C        ludcmp stands for LU (lower/upper triangular) decomposition.
C        Numerical Recipes, page 35, beautiful documentation.
C        The kernel of the good matrix inversion/solving methods.
C
C     Constantijn Panis,  3 December 2002
C     Last change: vv now argument
 
      implicit none
      integer n
      double precision a(n,n),vv(n),dum,sum,aamax,sign,eps
      integer status,indx(n),i,j,k,imax
      parameter(eps=1.d-20)
 
      sign=1.d0
      do 120 i=1,n
         aamax=0.d0
         do 110 j=1,n
            if(dabs(a(i,j)).gt.aamax)aamax=dabs(a(i,j))
  110    continue
         if(aamax.eq.0.d0)goto 899
         vv(i)=1.d0/aamax
  120 continue
      do 190 j=1,n
         do 140 i=1,j-1
            sum=a(i,j)
            do 130 k=1,i-1
               sum=sum-a(i,k)*a(k,j)
  130       continue
            a(i,j)=sum
  140    continue
         aamax=0.d0
         do 160 i=j,n
            sum=a(i,j)
            do 150 k=1,j-1
               sum=sum-a(i,k)*a(k,j)
  150       continue
            a(i,j)=sum
            dum=vv(i)*dabs(sum)
            if(dum.ge.aamax)then
               imax=i
               aamax=dum
            endif
  160    continue
         if(j.ne.imax)then
            do 170 k=1,n
               dum=a(imax,k)
               a(imax,k)=a(j,k)
               a(j,k)=dum
  170       continue
            sign=-sign
            vv(imax)=vv(j)
         endif
         indx(j)=imax
         if(a(j,j).eq.0.d0)a(j,j)=eps
         if(j.ne.n)then
            dum=1.d0/a(j,j)
            do 180 i=j+1,n
               a(i,j)=a(i,j)*dum
  180       continue
         endif
  190 continue
      goto 999
 
  899 status=1
  999 return
      end
c=====================================================================
      subroutine lubksb(a,n,indx,b)
C Inputs:
C        a(n,n) is input, as its LU Decomposition (see LUDCMP).
C        Indx is the permutation vector produced by LUDCMP.
C        B is input as the rhs vector B
C Outputs:
C        B returns with the solution vector X.
C        A, N, NP, INDX are not altered.
C Function:
C        lubksb stands for LU backward substitution of b.
C        Solves set of N equations A.X = B, in conjunction with LUDCMP
C        Numerical Recipes, page 36, beautiful documentation
C        Written by Ajay Shah;
C        adapted by Constantijn Panis,  8 August 1990
C*
      implicit none
      integer n
      double precision a(n,n),b(n),sum
      integer indx(n),i,ii,j,ll
 
      ii=0
      do 120 i=1,n
         ll=indx(i)
         sum=b(ll)
         b(ll)=b(i)
         if(ii.ne.0)then
            do 110 j=ii,i-1
               sum=sum-a(i,j)*b(j)
  110       continue
         elseif(sum.ne.0.d0)then
            ii=i
         endif
         b(i)=sum
  120 continue
      do 140 i=n,1,-1
         sum=b(i)
         do 130 j=i+1,n
            sum=sum-a(i,j)*b(j)
  130    continue
         b(i)=sum/a(i,i)
  140 continue
      return
      end
