      subroutine writedble(unit,x,skip,n,maxm,lf)
 
c     Subroutine to write double precision number x in "best" format.
c     Depending on the value of x and the number of spaces available
c     for writing it, the best format may be fixed ("F") or
c     scientific ("1PE").  If x is integer-valued, an attempt is made
c     to use a fixed format with one digit (0) after the decimal point.
c     While this is a waste of space, omitting the zero would often be
c     confusing.  Arguments:
c
c        unit [in] unit to which integer needs to be written
c        x    [in] double precision number to be written
c        skip [in] related to the number of spaces to be skipped before
c                  writing x:  if >=0, write skip blanks
c                              if <0 and x<0, write abs(skip)-1 blanks
c                              if <0 and x>=0, write abs(skip) blanks
c        n    [in] related to the number of spaces available for writing x
c                  (excluding additional spaces as specified by skip.)
c                  if n>0, use n spaces.  Note that if x>=0 and skip<0,
c                     one blank is written and only n-1 spaces are used.
c                     This is useful for neatly lining up numbers.
c                     Trailing zeroes are replaced by leading blanks.
c                  if n<0, use abs(n) spaces, unless fixed format is
c                     best, in which case leading blanks and trailing
c                     zeroes are removed.  (If maxm>0, trailing zeroes
c                     are not affected.)
c        maxm [in] maximum number of digits after the decimal point; only
c                  applicable if fixed format is best.  To disable such
c                  maximum, call with maxm=0.
c        lf   [in] integer controlling whether a linefeed is written after
c                  the number:  0=no;  other=yes.
c
c     Note:  small n may result in ugly results if the number is too
c     large or small for the "f" format.  The "1pe" format needs 5
c     spaces for the decimal point and the exponent if x>0 (e.g., 1.2E-04),
c     and 6 spaces if x<0 (e.g., -1.2E-04).  So if x>0, make sure n>=6,
c     and if x<0, n>=7.  Failure to do so results in the fixed format,
c     which may yield 0. if abs(x)<1 and asterisks if abs(x)>1.
c
c     Special arrangements are made for quiet NaN (detected by x!=x),
c     infinity (defined by bit pattern '7777777777777777') and not
c     applicable (defined by bit pattern '7777777777777776')
c
c     Constantijn Panis, 3 June 1996
 
      implicit none
      integer unit,skip,n,maxm,lf,ss,m,an,nsd,i,dlen,trm
      double precision x,ax,dn,tmp
      character i2c*12,fmt*16,ssc*3,anc*3,mc*2
      logical isnan,isinf,isna
 
c     Make sure nothing is wildly illegal:
      if(unit.lt.0)goto 999
      if(abs(n).le.2)goto 999
      if(maxm.lt.0)goto 999
 
      an=abs(n)
      ss=abs(skip)
      if(skip.lt.0.and.x.lt.0.d0)ss=ss-1
      if(skip.lt.0.and.x.ge.0.d0)an=an-1
      dn=dble(an)
 
      if(isnan(x).or.isinf(x).or.isinf(-x).or.isna(x))then
c        x is NaN, Infinity, -Infinity, or N/A
         if(n.lt.0)then
            if(isnan(x))then
               an=min(an,3)
            elseif(isinf(x))then
               an=min(an,8)
            elseif(isinf(-x))then
               an=min(an,9)
            elseif(isna(x))then
               an=min(an,3)
            endif
         endif
         ssc=i2c(ss,0)
         anc=i2c(an,0)
c        fmt is something like '(4x,a3)'
         fmt='('
         if(ss.gt.0)fmt=fmt(1:1)//ssc(1:trm(ssc))//'x,'
         fmt=fmt(1:trm(fmt))//'a'//anc(1:trm(anc))//')'
         if(lf.eq.0)fmt=fmt(1:trm(fmt)-1)//',$)'
c        if(lf.eq.0)fmt=fmt(1:trm(fmt)-1)//',$)'
         if(isnan(x))then
            write(unit,fmt)'NaN'
         elseif(isinf(x))then
            write(unit,fmt)'Infinity'
         elseif(isinf(-x))then
            write(unit,fmt)'-Infinity'
         elseif(isna(x))then
            write(unit,fmt)'N/A'
         endif
         goto 999
      endif
 
      ax=abs(x)
 
c     If number is integer-valued and small enough to fit in the
c     allotted space, write out as integer with ".0" attached.
c     This ".0" prevents confusion in reading such numbers; one
c     easily interprets "." as the end of a sentence.  There's one
c     exception in case there is insufficient space for ".0"; then
c     the "0" is suppressed.
      if(x.eq.dint(x).and.-10.d0**(dn-2).lt.x.and.x.lt.10.d0**(dn-1))
     *   then
         if(n.lt.0)an=min(an,dlen(x)+2)
c        In principle trailing ".0", unless pressed for space:
         m=min(1,an-dlen(x)-1)
         if(x.eq.0.d0)then
            if(n.lt.0)an=min(an,2)
            m=min(1,an-1)
         endif
         anc=i2c(an,0)
         mc=i2c(m,0)
         if(ss.eq.0)then
c           fmt is something like '(f6.1)'
            fmt='(f'//anc(1:trm(anc))//'.'//mc(1:trm(mc))//')'
         else
c           Could add ss to an, but this would lead the program to
c           use the additional space in case of numbers that are
c           too large, instead of writing out asterisks.
            ssc=i2c(ss,0)
c           fmt is something like '(4x,f6.1)'
            fmt='('
            if(ss.gt.0)fmt=fmt(1:1)//ssc(1:trm(ssc))//'x,'
            fmt=fmt(1:trm(fmt))//'f'//anc(1:trm(anc))//'.'//
     *         mc(1:trm(mc))//')'
         endif
         goto 200
      endif
 
c     The number is not (small enough) integer-valued.
c     Compute number of significant digits if fixed format were used.
c     No need to do so if there is insufficient space for scientific
c     notation; then fixed is always chosen.
      if((x.lt.0.d0.and.an.le.6).or.(x.ge.0.d0.and.an.le.5))then
c        Insufficient space for scientific notation.
         continue
      elseif(x-.5d0.le.-10.d0**(dn-2))then
c        Number is too large for the f format.
         nsd=0
      elseif(x.le.-1.d0)then
c        Two spaces lost for sign and decimal point:
         nsd=an-2
c        But not all spaces may be utilized if maxm>0.
         if(maxm.gt.0)nsd=min(nsd,maxm+dlen(x)-1)
      elseif(x.lt.0.d0)then
c        Spaces lost for sign, decimal point, and zeroes: -0.00123
c        Number of zeroes after decimal point is -int(dlog(ax)).
         nsd=an-2+int(dlog10(ax))
c        But not all spaces may be utilized if maxm>0.
         if(maxm.gt.0)nsd=max(0,maxm+int(dlog10(ax)))
      elseif(x.lt.1.d0)then
c        Spaces lost for decimal point and zeroes: 0.00123
c        Number of zeroes after decimal point is -int(dlog(ax)).
         nsd=an-1+int(dlog10(ax))
c        But not all spaces may be utilized if maxm>0.
         if(maxm.gt.0)nsd=max(0,maxm+int(dlog10(ax)))
      elseif(x+.5d0.le.10.d0**(dn-1))then
c        The largest positive value that may be written out by the f format
c        is 10**(n-1), exclusive.  Note that x will be rounded.
c        One space lost for decimal point:
         nsd=an-1
c        But not all spaces may be utilized if maxm>0.
         if(maxm.gt.0)nsd=min(nsd,maxm+dlen(x))
      else
c        Number is too large for fixed format.
         nsd=0
      endif
 
c     Scientific notation loses five spaces for the decimal point and
c     the exponent; if negative, one extra for the sign.  If there is
c     insufficient space for scientific notation, also select fixed
c     format, which may result in '****' upon output.
      if((x.lt.0.d0.and.(an.le.6.or.nsd.ge.an-6)).or.
     *   (x.ge.0.d0.and.(an.le.5.or.nsd.ge.an-5)))then
c        Fixed format, f<an>.<m>:
         if(x.le.-1.d0)then
            m=an-1-dlen(x)
         elseif(x.lt.0.d0)then
            m=an-2
         elseif(x.lt.1.d0)then
            m=an-1
         else
            m=an-1-dlen(x)
         endif
         m=max(m,0)
         if(maxm.ne.0.and.m.gt.maxm)then
c           Number of digits behind the decimal point is limited;
c           overall width of number may be decreased, if desired.
            if(n.lt.0)an=an-(m-maxm)
            m=maxm
         else
c           Strip trailing zeroes (only if maxm=0; don't strip first zero):
            do 110 i=m,2,-1
               tmp=(ax-dint(ax))*10.d0**dble(i)
               if(abs(tmp-10.d0*dint(.1d0*tmp+.5d0)).gt..5d0)goto 120
  110       continue
c           Normally, trailing zeroes are replaced by leading blanks;
c           if desired, overall width of number is decreased instead.
  120       if(n.lt.0)an=an-(m-i)
            m=i
         endif
         anc=i2c(an,0)
         mc=i2c(m,0)
         if(ss.eq.0)then
c           fmt is something like '(f12.4)'
            fmt='(f'//anc(1:trm(anc))//'.'//mc(1:trm(mc))//')'
         else
c           Could add ss to an, but this would lead the program to
c           use the additional space in case of numbers that are
c           too large, instead of writing out asterisks.
            ssc=i2c(ss,0)
c           fmt is something like '(6x,f12.4)'
            fmt='('
            if(ss.gt.0)fmt=fmt(1:1)//ssc(1:trm(ssc))//'x,'
            fmt=fmt(1:trm(fmt))//'f'//anc(1:trm(anc))//'.'//
     *         mc(1:trm(mc))//')'
         endif
      else
c        Scientific notation
         if(x.lt.0.d0)then
            m=max(an-7,0)
         else
            m=max(an-6,0)
         endif
c        May add ss to an, because sufficient space is guaranteed
         an=an+ss
         anc=i2c(an,0)
         mc=i2c(m,0)
c        fmt is something like '(1pe12.4)'
         fmt='(1pe'//anc(1:trm(anc))//'.'//mc(1:trm(mc))//')'
      endif
 
c     Write out:
  200 if(lf.eq.0)fmt=fmt(1:trm(fmt)-1)//',$)'
c 200 if(lf.eq.0)fmt=fmt(1:trm(fmt)-1)//',$)'
      write(unit,fmt)x
 
  999 return
      end
